<?php

namespace Drupal\connectorg_extra_functions\Plugin\rest\resource;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\rest\Annotation\RestResource;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Represents Stars records as resources.
 *
 * @RestResource (
 *   id = "connectorg_extra_functions_stars",
 *   label = @Translation("Stars rest api"),
 *   uri_paths = {
 *     "canonical" = "/api/v1/connectorg_extra_functions_stars/create",
 *     "create" = "/api/v1/connectorg_extra_functions_stars/create"
 *   }
 * )
 */
class StarsResource extends ResourceBase {

  /**
   * An entity type manager instance.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a Drupal\rest\Plugin\rest\resource\EntityResource object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   A entity type manager instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    EntityTypeManagerInterface $entityTypeManager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('connectorg_extra_functions_rest'),
      $container->get("entity_type.manager")
    );
  }

  /**
   * Responds to POST requests and saves the new record.
   *
   * @param mixed $data
   *   Data to write into the database.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   */
  public function post($data): ModifiedResourceResponse {
    try {
      /** @var \Drupal\user\Entity\User $employee */
      $employee = $this->findUserByEmail($data["employee"]);
      /** @var \Drupal\user\Entity\User $sentBy */
      $sentBy = $this->findUserByEmail($data["sentby"]);
      $term = $this->entityTypeManager->getStorage("taxonomy_term")
        ->loadByProperties([
          "vid" => "star_types",
          "name" => $data["type"]
        ]);
      $type = reset($term);
      $entity = NULL;
      if ($employee && $sentBy && $type) {
        $entity = $this->entityTypeManager->getStorage("star_entity")
          ->create([
            "field_employee" => $employee->id(),
            "field_message" => $data['message'],
            "field_type" => $type->id(),
            "user_id" => $sentBy->id(),
            "created" => $data['created'],
          ]);
        $entity->save();
      }

      return new ModifiedResourceResponse($entity ? $entity->toArray() : [], 201);
    }
    catch (InvalidPluginDefinitionException | EntityStorageException | PluginNotFoundException $e) {
      $this->logger->error($e->getTraceAsString());
      return new ModifiedResourceResponse($data, 500);
    }
  }

  /**
   * Auxiliary method to get user by email.
   *
   * @param mixed $email
   *   Email.
   *
   * @return \Drupal\Core\Entity\EntityInterface|false|null
   *   User entity.
   */
  protected function findUserByEmail($email) {
    try {
      $user = $this->entityTypeManager->getStorage("user")
        ->loadByProperties([
          "mail" => $email,
        ]);
      return reset($user);
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      $this->logger->error($e->getTraceAsString());
      return NULL;
    }
  }

}
